import boto3
import click
import logging
import time, sys
from termcolor import colored


def check_tagging(bucket):
    has_confirmation = False
    s3 = boto3.client( 's3' )
    tag = s3.get_bucket_tagging(Bucket=bucket)
    tags = tag.get('TagSet')
    for x in tags:
        if x.get('Key') == 's3bro_delete':
            if x.get('Value') == "yes":
                has_confirmation = True
    return has_confirmation


def delete_confirmation(bucket, prefix):
    avoid_confirmation = check_tagging(bucket)
    if avoid_confirmation is True:
        clean_bucket(bucket, prefix)
    else:
        if prefix is '':
            print_prefix = '/'
        else:
            print_prefix = prefix
        print(colored('[Warning] This action is not reversible', 'red'))
        print(30*'=')
        print('Bucket: %s' % bucket)
        print('Prefix: %s' % print_prefix)
        print(30*'=')
        r = raw_input("Confirm the bucket name if you want to wipe the bucket content: ")
        if r == bucket:
            clean_bucket(bucket, prefix)
        else:
            print('something is not right with the confirmation')


def clean_bucket(bucket, prefix):
    s3 = boto3.resource( 's3' )
    click.echo('I will start the deletion of %s/%s in 10 seconds, you still have the chance to stop' % (bucket, prefix))
    for remaining in range( 10, 0, -1 ):
        sys.stdout.write( "\r" )
        sys.stdout.write( "{:2d} seconds remaining.".format( remaining ) )
        sys.stdout.flush()
        time.sleep( 1 )
    click.echo('\nStart cleaning...')
    bkt = s3.Bucket(bucket)
    iterator = bkt.object_versions.filter( Prefix=prefix )
    objects = []
    for obj in iterator:
        objects.append( {'Key': obj.key, 'VersionId': obj.id} )
        logging.warning('Sending to deletion: %s %s' % (obj.key, obj.id))
        if len(objects) == 1000:
            response = bkt.delete_objects(Delete={'Objects': objects})
            objects = []
    if len(objects) > 0:
        click.echo('You have only %s keys, deleting'%len(objects))
        response = bkt.delete_objects( Delete={'Objects': objects} )
    else:
        print('it seem that you got no keys')




import boto3
import click
import logging
from pool_map import multi_process
from botocore.exceptions import ClientError


def is_encrypted(obj):
    enc = obj.get('ResponseMetadata').get('HTTPHeaders').get('x-amz-server-side-encryption')
    if not enc:
        return False
    else:
        return True


def get_encryption(obj):
    s3 = boto3.resource('s3')
    try:
        r = s3.ObjectVersion(obj.get('bucket'), obj.get('key'), obj.get('version')).head()
        if not is_encrypted(r):
            print('[Not Encrypted] %s | %s' % (obj.get('key'), obj.get('version')))
        else:
            logging.warning('[Encrypted] %s | %s' % (obj.get('key'), obj.get('version')))

    except ClientError as e:
        if e.response['ResponseMetadata']['HTTPStatusCode'] == 405:
            logging.warning('[Delete Marker Found] - Ignoring key %s/%s VersionID: %s' % (obj.get('bucket'), obj.get('key'), obj.get('version')))
        elif e.response['ResponseMetadata']['HTTPStatusCode'] == 403 or e.response['ResponseMetadata']['HTTPStatusCode'] == 404:
            print('Failed to retrieve encryption details %s %s (Access Denied)' % (obj.get('key'), obj.get('version')))
        elif e.response['ResponseMetadata']['HTTPStatusCode'] == 503:
            print('SlowDown - Too many requests. Failed to retrieve %s | %s' % (obj.get('bucket'), obj.get('key'), obj.get('version')))
        else:
            logging.warning(obj.get('bucket'), obj.get('key'), obj.get('version'), e.response)


def find_unencrypted_keys(bucket, prefix, versions, workers):
    s3 = boto3.resource( 's3' )
    bkt = s3.Bucket( bucket )
    objects = []
    if versions:
        iterator = bkt.object_versions.filter(Prefix=prefix)
    else:
        iterator = bkt.objects.filter( Prefix=prefix )

    processed = False
    for k in iterator:
        processed = False
        if len( objects ) < 1000:
            if not versions:
                data = {'bucket': bucket, 'key': k.key, 'version': 'null'}
            else :
                data = {'bucket': bucket, 'key': k.key, 'version': k.id}
            objects.append(data)
            # objects.append([bucket, k.key]) if not versions else objects.append([bucket, k.key, k.id])
        else:
            multi_process(get_encryption, objects, workers)
            del objects[:]
            processed = True
    if not processed:
        multi_process(get_encryption, objects, workers)
